@@ -866,7 +866,7 @@ on_show_search (GtkAction *action, gpointer user_data)
 	if (!gtk_widget_get_parent (search_box))
 		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
 
-	search_box_fill_search_focus (SEARCH_BOX (search_box));
+	search_box_fill_search_focus (SEARCH_BOX (search_box), FALSE);
 	gtk_widget_show (search_box);
 }
 
@@ -884,7 +884,116 @@ on_repeat_quicksearch (GtkAction *action, gpointer user_data)
 
 	if (!gtk_widget_get_visible (search_box))
 		gtk_widget_show (search_box);
-	on_search_activated (NULL, SEARCH_BOX (search_box));
+	search_box_incremental_search (SEARCH_BOX (search_box), TRUE);
+}
+
+void
+on_search_previous (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	if (!gtk_widget_get_visible (search_box))
+		gtk_widget_show (search_box);
+	search_box_incremental_search (SEARCH_BOX (search_box), FALSE);
+}
+
+void 
+on_search_and_replace (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	search_box_fill_search_focus (SEARCH_BOX (search_box), TRUE);
+	gtk_widget_show (search_box);
+}
+
+void 
+on_search_popup_case_sensitive_toggle (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	gboolean case_sensitive_status = 
+		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	if (!gtk_widget_get_visible (search_box))
+		gtk_widget_show (search_box);
+	
+	search_box_toggle_case_sensitive (SEARCH_BOX (search_box), case_sensitive_status);
+}
+
+void 
+on_search_popup_highlight_toggle (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	gboolean highlight_status = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));	
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	if (!gtk_widget_get_visible (search_box))
+		gtk_widget_show (search_box);
+
+	search_box_toggle_highlight (SEARCH_BOX (search_box), highlight_status);
+}
+
+void
+on_search_popup_clear_highlight (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	if (!gtk_widget_get_visible (search_box))
+		gtk_widget_show (search_box);
+
+	search_box_clear_highlight (SEARCH_BOX (search_box));
+}
+
+void
+on_search_popup_regex_search (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	GtkWidget *search_box;
+
+	gboolean regex_active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+
+	search_box = plugin->search_box;
+	if (!gtk_widget_get_parent (search_box))
+		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
+
+	if (!gtk_widget_get_visible (search_box))
+		gtk_widget_show (search_box);
+
+	search_box_toggle_regex (SEARCH_BOX (search_box), regex_active);
 }
 
 void
