@@ -54,6 +54,8 @@
 #include "git-reset-pane.h"
 #include "git-revert-pane.h"
 #include "git-cherry-pick-pane.h"
+#include "git-patch-series-pane.h"
+#include "git-apply-mailbox-pane.h"
 
 AnjutaCommandBarEntry branch_entries[] =
 {
@@ -84,7 +86,7 @@ AnjutaCommandBarEntry branch_entries[] =
 	{
 		ANJUTA_COMMAND_BAR_ENTRY_BUTTON,
 		"Switch",
-		N_("Switch to this branch"),
+		N_("Switch to the selected branch"),
 		N_("Switch to the selected branch"),
 		GTK_STOCK_JUMP_TO,
 		G_CALLBACK (on_switch_branch_button_clicked)
@@ -405,6 +407,62 @@ static AnjutaCommandBarEntry log_entries[] =
 		N_("Revert a commit"),
 		GTK_STOCK_UNDO,
 		G_CALLBACK (on_revert_button_clicked)
+	},
+	{
+		ANJUTA_COMMAND_BAR_ENTRY_FRAME,
+		"NULL",
+		N_("Patch series"),
+		NULL,
+		NULL,
+		NULL
+	},
+	{
+		ANJUTA_COMMAND_BAR_ENTRY_BUTTON,
+		"GeneratePatchSeries",
+		N_("Generate a patch series"),
+		N_("Generate a patch series"),
+		NULL,
+		G_CALLBACK (on_patch_series_button_clicked)
+	},
+	{
+		ANJUTA_COMMAND_BAR_ENTRY_FRAME,
+		"NULL",
+		N_("Mailbox files"),
+		NULL,
+		NULL,
+		NULL
+	},
+	{
+		ANJUTA_COMMAND_BAR_ENTRY_BUTTON,
+		"ApplyMailboxFiles",
+		N_("Apply mailbox files"),
+		N_("Apply patches from mailbox files"),
+		NULL,
+		G_CALLBACK (on_apply_mailbox_button_clicked)
+	},
+	{
+		ANJUTA_COMMAND_BAR_ENTRY_BUTTON,
+		"ApplyMailboxContinue",
+		N_("Continue"),
+		N_("Continue applying patches with resolved conflicts"),
+		NULL,
+		G_CALLBACK (on_apply_mailbox_continue_button_clicked)
+	},
+	{
+		ANJUTA_COMMAND_BAR_ENTRY_BUTTON,
+		"ApplyMailboxSkip",
+		N_("Skip"),
+		N_("Skip the current patch in the series"),
+		NULL,
+		G_CALLBACK (on_apply_mailbox_skip_button_clicked)
+	},
+	{
+		ANJUTA_COMMAND_BAR_ENTRY_BUTTON,
+		"ApplyMailboxAbort",
+		N_("Abort"),
+		N_("Stop applying the patch series and return the tree to its previous state"),
+		NULL,
+		G_CALLBACK (on_apply_mailbox_abort_button_clicked)
 	}
 };
 
@@ -830,6 +888,12 @@ git_class_init (GObjectClass *klass)
 	klass->dispose = git_dispose;
 }
 
+
+void git_plugin_status_changed_emit(AnjutaCommand *command, guint return_code, Git *plugin)
+{
+        g_signal_emit_by_name(plugin, "status-changed");
+}
+
 ANJUTA_PLUGIN_BEGIN (Git, git);
 ANJUTA_PLUGIN_ADD_INTERFACE (git_ivcs, IANJUTA_TYPE_VCS);
 ANJUTA_PLUGIN_END;
