@@ -917,7 +917,8 @@ on_insert_button_clicked (GtkButton *insert_button,
 
 		snippets_interaction_insert_snippet (priv->snippets_interaction,
 		                                     priv->snippets_db,
-		                                     ANJUTA_SNIPPET (cur_object));
+		                                     ANJUTA_SNIPPET (cur_object),
+		                                     TRUE);
 	}
 
 }
@@ -969,7 +970,8 @@ on_snippets_view_row_activated (GtkTreeView *snippets_view,
 	if (ANJUTA_IS_SNIPPET (cur_object))
 		snippets_interaction_insert_snippet (priv->snippets_interaction,
 		                                     priv->snippets_db,
-		                                     ANJUTA_SNIPPET (cur_object));
+		                                     ANJUTA_SNIPPET (cur_object),
+		                                     TRUE);
 
 	g_object_unref (cur_object);
 }
